/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
******************************************************************************
**                                                                            
**        Y     Y   OOOO    OOOO   DDDDD     AA
**         Y   Y   O    O  O    O  D    D   A  A
**          Y Y    O    O  O    O  D    D  A    A 
**           Y     O    O  O    O  D    D  AAAAAA
**           Y     O    O  O    O  D    D  A    A
**           Y      OOOO    OOOO   DDDDD   A    A
**
******************************************************************************
******************************************************************************/
/*****************************************************************************
*
*  MODULE :             Database server
*  SCOPE :              Database server
*  FILE :               dbserver.hxx
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma interface
#endif
#ifndef __dbserver_HXX
#define __dbserver_HXX

/* IMPORTS *******************************************************************/

#ifndef __stdlib_H
#define __stdlib_H
#include <stdlib.h>
#endif

#ifndef __stdio_H
#define __stdio_H
#include <stdio.h>
#endif

#ifndef __string_H
#define __string_H
#include <string.h>
#endif

#ifndef __class_HXX
#include "class.hxx"
#endif

#ifndef __config_H
#include "config.h"
#endif

#ifndef __yootypes_H
#include "yootypes.h"
#endif

#ifndef __pagehdl_HXX
#include "pagehdl.hxx"
#endif

#ifndef __list_HXX
#include "list.hxx"
#endif

/*****************************************************************************/

class DBServer {

  class PageHdlLock : public PageHdl {
    private :

    Lock _locktable[MAX_VUE] ; // Table of locks

    public :

    PageHdlLock(const PID& pid) ; // Build a page handler for lock management
    Lock lock(int vue) ;       // Return lock hold by the vue
    int setLock(int, Lock) ;   // Attempt to set a lock for a DBVue
    int unlock(int) ;          // Unlock page. Return 0 if no more lock
    void upgradeLock(int,Lock);// Upgrade lock for a vue
    static int checklock(Lock, Lock); // Return TRUE if locks are compatible
  } ;
  
  private :

  char* _name ;              // Name of the server in the database
  char* _log ;               // Directory where this server creates logs
  Volume* _tabvol[MAX_VOL] ; // Array of volumes this server manages
  ServerDBVue* _tabvue[MAX_VUE] ;  // Array of dbvue connected on this server
  PageSet _lock ;            // HEM set of locked pages identified by PID
  
  public :

  DBServer(const char* name) ; // Build the server by its name
  ~DBServer() ;
  void setLog(char* l) ;     // Set the directory to create log
  void setVolume(ListPtr<Volume>*); // Set the directory to create log
  char* name() ;             // Return the name of the server
  Volume* volume(int i) ;    // return null or throw exceptions
  int registerVue(ServerDBVue*) ;  // Register a new vue in the server
  void keepLocks(int,ListVal<PID>&,ListVal<PID>&,ListVal<PID>&); 
  void releaseLocks(int, ListVal<PID>&) ; // Release locks for a vue
  int setLock(int, const PID&, Lock); // attempt to lock a page for a dbvue
  Lock lock(int, const PID&) ;     // Return the lock hold by the dbvue
  void unlock(int, const PID&);    // unlock a page for a dbvue
  char* createLogName(int) ; // Create a log name for a vue
  void cleanLog() ;          // Find unfinished log and do undo
  void undo(Log*) ;          // Undo aborted transaction from a log
  int size() ;               // Return the number of volumes handle by server
} ;


inline void DBServer::setLog(char* l) {
  if (this->_log != NULL) delete this->_log ;
  this->_log = strdup(l) ;
}

inline char* DBServer::createLogName(int vue) {
  char* logfile = new char[256] ;
  sprintf(logfile,"%s/.yoo_%s_%d_log", this->_log, this->_name, vue) ;
  return logfile ;
}

inline char* DBServer::name() {
  return _name ;
}

inline Volume* DBServer::volume(int i) {
  return _tabvol[i] ;
}

inline Lock DBServer::lock(int vue, const PID& pid) {
  PageHdlLock* pg = (PageHdlLock*)_lock[pid] ;
  return pg == NULL ? YOO_UNLOCK : pg->lock(vue) ;
}

inline DBServer::PageHdlLock::PageHdlLock(const PID& pid) : PageHdl(pid) {
  for (int i = 0; i < MAX_VUE; _locktable[i++] = YOO_UNLOCK) ;
}

inline Lock DBServer::PageHdlLock::lock(int vue) {
  return this->_locktable[vue] ;
}

inline void DBServer::PageHdlLock::upgradeLock(int vue, Lock l) {
  this->_locktable[vue] = l ;
}  

inline int DBServer::size() {
  int count = 0 ;
  for (int i = 0; i < MAX_VOL; i++) {
    if (_tabvol[i] != NULL) count++ ;
  }
  return count ;
}  

#endif
